==> zen-cortex-8.0.3.tgz: package/api/fetch.js <==
import { encodeToQueryString } from "../actions/router";

/** HTTP FETCH implementation
  see https://fetch.spec.whatwg.org/
  @param {string} uri - full URL endpoint
  @param {string} method - HTTP verb
  @param {object} payload-  to send to server
  @param {object} [options] - options to customize XHR
  @param {string} token - JWT session token
  @returns {promise} thenable
*/
export default function fetcher(
  uri,
  method = "GET",
  payload = null,
  options = {},
  token = ""
) {
  //default JSON-request options
  const settings = {
    method,
    headers: {}
  };

  //add token if provided
  if (token) {
    settings.headers["Authorization"] = `Bearer ${token}`;
  }

  //proxy to native or polyfilled fetch (https://github.com/github/fetch)
  let fetchMethod = window.fetch;
  let queryURL = "";
  if (payload) {
    if (method === "GET") {
      //payload is added to the url query string for GET requests.
      queryURL = encodeToQueryString(payload);
    } else if (payload instanceof FormData) {
      //just use form as body but with customized fetch
      settings.body = payload;
      fetchMethod = customizedFetch;
    } else {
      //otherwise use stringified body
      settings.headers["Content-Type"] = "application/json";
      settings.body = JSON.stringify(payload);
    }
  }
  return fetchMethod(uri + queryURL, settings, options).then(handleResponse);
}

/**
 * Check and manage HTTP response
 * @param {promise} response
 */
function handleResponse(response) {
  const { status } = response;
  if (status >= 200 && status < 300) {
    //no-content
    if (status === 204) return true;
    //parse it
    return parseResponse(response);
  }
  //otherwise exception
  return throwException(response);
}

/**
 * Read response as JSON result.
 * @param {promise|xhr} response
 */
function parseResponse(response) {
  return response.json ? response.json() : JSON.parse(response.responseText);
}

/**
 * Build standardized exception as rejected response
 * @param {promise|xhr} response
 */
function throwException(response) {
  //helper to build generic response exception
  function buildException(message, content) {
    let exc = new Error(message);
    exc.number = response.status;
    exc.content = content;
    return exc;
  }

  if (response.json) {
    return response.json().then(json => {
      //case body payload is just a string message
      if (typeof json === "string") throw buildException(json);
      throw buildException(response.statusText, json);
    });
  }
  throw buildException(response.responseText);
}

/**
  Promisify a API fetch call using given options.
  @return {promise} that encapsulate an enhanced XMLHttpRequest instance.
*/
function customizedFetch(url, settings = {}, { onProgress }) {
  return new Promise((resolve, reject) => {
    const xhr = new XMLHttpRequest();
    xhr.open(settings.method, url);
    //copy headers
    for (let h in settings.headers) {
      xhr.setRequestHeader(h, settings.headers[h]);
    }
    xhr.onload = e => resolve(e.target);
    if (xhr.upload && onProgress) {
      //register progress callback
      xhr.upload.onprogress = evt => {
        //compute progress and notify
        if (evt.lengthComputable) {
          onProgress(Math.floor((evt.loaded / evt.total) * 100));
        }
      };
    }
    xhr.onerror = reject;
    xhr.send(settings.body);
  });
}

==> zen-cortex-8.0.4.tgz: package/api/fetch.js <==
import { nls } from "zen-lingual";
import { encodeToQueryString } from "../actions/router";

/** HTTP FETCH implementation
  see https://fetch.spec.whatwg.org/
  @param {string} uri - full URL endpoint
  @param {string} method - HTTP verb
  @param {object} payload-  to send to server
  @param {object} [options] - options to customize XHR
  @param {string} token - JWT session token
  @returns {promise} thenable
*/
export default function fetcher(
  uri,
  method = "GET",
  payload = null,
  options = {},
  token = ""
) {
  //default JSON-request options
  const settings = {
    method,
    headers: {}
  };

  //add token if provided
  if (token) {
    settings.headers["Authorization"] = `Bearer ${token}`;
  }

  //proxy to native or polyfilled fetch (https://github.com/github/fetch)
  let fetchMethod = window.fetch;
  let queryURL = "";
  if (payload) {
    if (method === "GET") {
      //payload is added to the url query string for GET requests.
      queryURL = encodeToQueryString(payload);
    } else if (payload instanceof FormData) {
      //just use form as body but with customized fetch
      settings.body = payload;
      fetchMethod = customizedFetch;
    } else {
      //otherwise use stringified body
      settings.headers["Content-Type"] = "application/json";
      settings.body = JSON.stringify(payload);
    }
  }
  return fetchMethod(uri + queryURL, settings, options).then(handleResponse);
}

/**
 * Check and manage HTTP response
 * @param {promise} response
 */
function handleResponse(response) {
  const { status } = response;
  if (status >= 200 && status < 300) {
    //no-content
    if (status === 204) return true;
    //parse it
    return parseResponse(response);
  }
  //otherwise exception
  return throwException(response);
}

/**
 * Read response as JSON result.
 * @param {promise|xhr} response
 */
function parseResponse(response) {
  return response.json ? response.json() : JSON.parse(response.responseText);
}

/**
 * Build standardized exception as rejected response
 * @param {promise|xhr} response
 */
function throwException(response) {
  //helper to build standardized exception
  function buildException(message, content) {
    let exc = new Error(message);
    exc.number = response.status;
    exc.content = content;
    return exc;
  }

  if (response.json) {
    return response.json().then(json => {
      //case body payload is just a string message
      if (typeof json === "string") throw buildException(json);
      throw buildException(response.statusText, json);
    });
  }
  throw buildException(response.responseText);
}

/**
  Promisify a API fetch call using given options.
  @return {promise} that encapsulate an enhanced XMLHttpRequest instance.
*/
function customizedFetch(url, settings = {}, { onProgress }) {
  return new Promise((resolve, reject) => {
    const xhr = new XMLHttpRequest();
    xhr.open(settings.method, url);
    //copy headers
    for (let h in settings.headers) {
      xhr.setRequestHeader(h, settings.headers[h]);
    }

    if (xhr.upload && onProgress) {
      //register progress callback
      xhr.upload.onprogress = evt => {
        //compute progress and notify
        if (evt.lengthComputable) {
          onProgress(Math.floor((evt.loaded / evt.total) * 100));
        }
      };
    }

    xhr.onload = e => resolve(e.target);
    xhr.onerror = () => reject(new Error(`3-413|${nls("core.error.upload")}`));
    xhr.send(settings.body);
  });
}
